from m3_gar.importer.table.consts import (
    PARAMS_SUFFIX,
)
from m3_gar.importer.table.exceptions import (
    ParentLookupError,
)
from m3_gar.models.util import (
    RegionCodeModelMixin,
)
from m3_gar.util import (
    get_model_from_table_name,
    get_table_row_filters,
)


class TableIterator:

    def __init__(self, table):
        self.table = table
        self.model = table.model

    def __iter__(self):
        if self.model is None:
            return []

        return self

    def get_context(self):
        raise NotImplementedError()

    def get_next(self):
        raise NotImplementedError()

    def format_row(self, row):
        raise NotImplementedError()

    def process_row(self, row):
        try:
            row = dict(self.format_row(row))
        except ParentLookupError as e:
            return None

        item = self.model(**row)

        if isinstance(item, RegionCodeModelMixin):
            item.region_code = self.table.region

        table_row_filters = get_table_row_filters()
        for filter_func in table_row_filters.get(self.model._meta.model_name, tuple()):
            item = filter_func(item)

            if item is None:
                break

        return item

    def __next__(self):
        return self.get_next()

    next = __next__


class Table:
    name = None
    iterator = TableIterator

    def __init__(self, *, filename, name, region, **kwargs):
        self.filename = filename
        self.region = region

        self.name = name.lower()
        self.model = get_model_from_table_name(self.name)

    def open(self, tablelist):
        return tablelist.open(self.filename)

    def rows(self, tablelist):
        raise NotImplementedError()



from m3_gar.base_models import (
    ChangeHistory as BaseChangeHistory,
    OperationTypes,
)
from m3_gar.models.util import (
    RegionCodeModelMixin,
    make_fk,
)


__all__ = ['ChangeHistory']


class ChangeHistory(BaseChangeHistory, RegionCodeModelMixin):
    """
    Сведения по истории изменений
    """
    class Meta:
        verbose_name = verbose_name_plural = 'История изменений'


make_fk(ChangeHistory, 'opertypeid', to=OperationTypes)



# Generated by Django 3.2.8 on 2021-10-29 12:12

import django.contrib.postgres.indexes
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('m3_gar', '0004_auto_20211013_1106'),
    ]

    operations = [
        migrations.AddIndex(
            model_name='addrobj',
            index=django.contrib.postgres.indexes.HashIndex(fields=['objectguid'], name='m3_gar_addr_objectg_b1f158_hash'),
        ),
        migrations.AddIndex(
            model_name='addrobj',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['isactive', 'isactual', 'level'], name='m3_gar_addr_isactiv_236c06_btree'),
        ),
        migrations.AddIndex(
            model_name='addrobjparams',
            index=django.contrib.postgres.indexes.HashIndex(fields=['objectid'], name='m3_gar_addr_objecti_335bbb_hash'),
        ),
        migrations.AddIndex(
            model_name='apartmentsparams',
            index=django.contrib.postgres.indexes.HashIndex(fields=['objectid'], name='m3_gar_apar_objecti_03b3f2_hash'),
        ),
        migrations.AddIndex(
            model_name='carplacesparams',
            index=django.contrib.postgres.indexes.HashIndex(fields=['objectid'], name='m3_gar_carp_objecti_12b2bc_hash'),
        ),
        migrations.AddIndex(
            model_name='housesparams',
            index=django.contrib.postgres.indexes.HashIndex(fields=['objectid'], name='m3_gar_hous_objecti_ad2270_hash'),
        ),
        migrations.AddIndex(
            model_name='reestrobjects',
            index=django.contrib.postgres.indexes.HashIndex(fields=['objectguid'], name='m3_gar_rees_objectg_281c48_hash'),
        ),
        migrations.AddIndex(
            model_name='roomsparams',
            index=django.contrib.postgres.indexes.HashIndex(fields=['objectid'], name='m3_gar_room_objecti_1f650d_hash'),
        ),
        migrations.AddIndex(
            model_name='steadsparams',
            index=django.contrib.postgres.indexes.HashIndex(fields=['objectid'], name='m3_gar_stea_objecti_66db3d_hash'),
        ),
    ]



# Generated by Django 3.2.4 on 2022-10-24 10:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('m3_gar', '0015_alter_normativedocs_orgname'),
    ]

    operations = [
        migrations.AlterField(
            model_name='normativedocs',
            name='orgname',
            field=models.CharField(blank=True, max_length=600, null=True, verbose_name='Наименование органа создвшего нормативный документ'),
        ),
    ]



import datetime

from m3_gar.importer.client import (
    client,
)
from m3_gar.importer.signals import (
    post_fetch_version,
    pre_fetch_version,
)
from m3_gar.models import (
    Version,
)


def get_or_create_version(version_id):
    dumpdate = datetime.datetime.strptime(str(version_id), "%Y%m%d").date()
    version = Version.objects.filter(
        ver=version_id,
    ).first()

    if version:
        ver = version

        if version.dumpdate < dumpdate:
            version.dumpdate = dumpdate
            version.save()
            created = True
        else:
            created = False
    else:
        ver = Version.objects.create(
            ver=version_id,
            dumpdate=dumpdate,
        )
        created = True

    return ver, created


def parse_item_as_dict(item, update_all=False):
    """
    Разбор данных о версии как словаря
    """
    ver, created = get_or_create_version(item['VersionId'])

    if created or update_all:
        setattr(ver, 'complete_xml_url', item.get('GarXMLFullURL', ''))
        setattr(ver, 'delta_xml_url', item.get('GarXMLDeltaURL', ''))

        ver.save()


def fetch_version_info(update_all=False):

    pre_fetch_version.send(object.__class__)

    versions = client.get_all_download_file_info()
    for item in versions:
        parse_item_as_dict(item=item, update_all=update_all)

    post_fetch_version.send(object.__class__)



# Generated by Django 2.2.28 on 2022-05-16 11:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('m3_gar', '0013_auto_20220513_0825'),
    ]

    operations = [
        migrations.AddField(
            model_name='addrobj',
            name='name_with_typename',
            field=models.CharField(max_length=300, null=True, verbose_name='Наименование c полным наименованием типа объекта'),
        ),
    ]



import sys


__all__ = ['nullcontext']


if sys.version_info >= (3, 10):
    from contextlib import (
        nullcontext,
    )
else:
    from contextlib import (
        AbstractAsyncContextManager,
        AbstractContextManager,
    )

    class nullcontext(AbstractContextManager, AbstractAsyncContextManager):
        """
        Бэкпорт contextlib.nullcontext из Python 3.10+
        """

        def __init__(self, enter_result=None):
            self.enter_result = enter_result

        def __enter__(self):
            return self.enter_result

        def __exit__(self, *excinfo):
            pass

        async def __aenter__(self):
            return self.enter_result

        async def __aexit__(self, *excinfo):
            pass



from m3_gar.base_models import (
    OperationTypes as BaseOperationTypes,
)


__all__ = ['OperationTypes']


class OperationTypes(BaseOperationTypes):
    """
    Сведения по статусу действия
    """
    class Meta:
        verbose_name = 'Статус действия'
        verbose_name_plural = 'Статусы действия'



import requests


class FIASClient:
    """Клиент к серверу ФИАС"""
    fias_source = 'https://fias.nalog.ru/WebServices/Public/'

    def get_all_download_file_info(self):
        result = requests.get(
            f'{self.fias_source}GetAllDownloadFileInfo',
        ).json()

        return result


client = FIASClient()
